#! BLCEL
# An application that can be run from bootstrap and receives
# the world as a parameter from the command line.

# It is intended for zipping with the world and then running
# as follows:
#
# export PYTHONPATH=_full_path_to_zipfile_.zip
# $(CEL)/celstart _full_path_to_world_.zip
# 
# although it can also run if celstrap.py and all other scripts are
# in the pythonpath (for example ($PWD)/scripts/)
# blender2crystal will be able to run using this from the RUN button.

# Usually you won't need to touch this file, but it is as easy to add
# (or remove) new functionality to it.
# The file needs to evolve

from pycel import *

# The class that implements the main application behaviour.
# if you want to make your own just copy it and set the same name for
# the file as for the class. ie: celstrap.py has class celstrap.

# To use this the player must have a camera property class and must be 
# called "camera" :(. Have to find a better way to manage the player.

class celstrap:
	api_version = 2
	def __init__(self,celEntity):
		print "Initializing game..."
		self.entity = celEntity
		# key bindings
		self.input = celCommandInput(self.entity)
		self.input.LoadConfig("Bootstrap")
		# create a tooltip for onscreen messages
		self.createTooltip()
		# create the tooltip for messages
		self.loadMovieRecorder()
		# Apricot: mount to shaders directory
		self.mountShaders()
		# go on and load the map
		self.map = "level.xml"
		self.loadMap(self.map)
		print "End initializing game..."

	def loadMovieRecorder(self):
		# load the movierecorder
		self.mr = oreg.Get(iMovieRecorder)
		if not self.mr:
			plugmgr = oreg.Get(iPluginManager)
			self.mr = CS_LOAD_PLUGIN (plugmgr, "crystalspace.utilities.movierecorder",iMovieRecorder)
			if self.mr:
				oreg.Register (self.mr, "iMovieRecorder")
			else:
				self.mr = None

	def createTooltip(self):
		self.logs=[]
		self.tooltip = celToolTip(self.entity)
		self.tooltip.SetBackgroundColor(-1,-1,-1)
		self.tooltip.SetTextColor(255,255,255)
		self.tooltip.Show(20,20)
		return
		self.log_event("celstart started")
		self.log_event("i: start record")
		self.log_event("o: stop record")
		self.log_event("p: pause record")

	def startScreen(self,map_file):
		font = Graphics2D.GetFontServer ().LoadFont("*large")
		if (not Graphics3D.BeginDraw (CSDRAW_2DGRAPHICS)):
			return
		Graphics2D.Clear(0)
		Graphics2D.Write(font,100,100,Graphics2D.FindRGB(255,255,255),Graphics2D.FindRGB(0,0,0),"Loading... "+str(map_file))
		Graphics2D.Print(csRect(0,0,640,480))
		Graphics3D.FinishDraw ()

	def mountShaders(self):
		if Vfs.Exists("/tmp/celstart/celstart.cfg"):
			destdir  = "/tmp/celstart/"
		else:
			destdir = "/this/"
		# Problem: Creating a symbolic link to a directory in a zip
		# does not work. So link thedirectory for the celstart package
		# to shaders. Ugly, but I see no other way.
		Vfs.SymbolicLink("/shader", destdir)

	def loadMap(self,map_file):
		self.startScreen(self.map)
		# find about right path
		zoneManager = celZoneManager(self.entity)
		if Vfs.Exists("/tmp/celstart/celstart.cfg"):
			destdir  = "/tmp/celstart"
		else:
			destdir = "/this/"
		Vfs.ChDirAuto(destdir);
		zoneManager.Load(destdir,map_file)
		self.startregion=zoneManager.LastStartRegionName
		region = zoneManager.FindRegion(self.startregion)
		zoneManager.ActivateRegion(region)
		
		# the following doesnt seem to be needed now
		# now the map is loaded we want to find the real
		# player entity and do some setup.
		# XXX ugly hardcoded name for the player
		# self.playername="camera"
		# self.startname=zoneManager.LastStartName
		# actor = Entities[self.playername]
		# if celGetMesh(actor):
		#	zoneManager.PointMesh(self.playername,self.startregion,self.startname)
		# else:
		#	zoneManager.PointCamera(self.playername,self.startregion,self.startname)

	def restart(self,pc=None,args=None):
		self.playername="camera"
		actor = self.entity.GetPropertyClassList().RemoveAll()
		self.loadMap(self.map)
	# video record commands
	def pccommandinput_startrecord1(self,pc,args):
		self.log_event("start recording")
		if self.mr:
			self.mr.Start()
	def pccommandinput_stoprecord1(self,pc,args):
		self.log_event("stop recording")
		if self.mr:
			self.mr.Stop()
	def pccommandinput_pauserecord1(self,pc,args):
		if self.mr:
			if self.mr.IsPaused():
				self.log_event("unpause recording")
				self.mr.UnPause()
			else:
				self.log_event("pause recording")
				self.mr.Pause()
	def log_event(self,event):
		self.logs.append(event)
		if len(self.logs) > 5:
			self.logs.pop(0)
		tooltip_text="\n".join(self.logs)
		self.tooltip.Show(20,20)
		self.tooltip.SetText(tooltip_text)

	# this are callbacks for pczonemanager (not used at the moment
	# as the zonemanager is a separate entity)
	#def pczonemanager_startloading(self,pc,args):
	#	self.log_event("start loading")
	#def pczonemanager_stoploading(self,pc,args):
	#	self.log_event("stop loading")
	#def pczonemanager_addregion(self,pc,args):
	#	self.log_event("add region")
	#def pczonemanager_remregion(self,pc,args):
	#	self.log_event("remove region")

