from pycel import *
from sequencer import FadedAction,TimedAction,MixerAction,RandomAction
from sequencer import ActionSequencer,TimedActionsManager,ActionsManager

class AnimeshWrapper(object):
	def __init__(self,obj):
		mw = self.FindMeshWrapper(obj)
		self.animesh = mw.MeshObject.QueryInterface(iAnimatedMesh)
		self.anifact = mw.MeshObject.Factory.QueryInterface(iAnimatedMeshFactory)
		self.skeleton = self.animesh.GetSkeleton()
		self.root = self.skeleton.GetAnimationPacket().GetAnimationRoot()
		self.protected = []
	def AddProtectedMorphTarget(self,target):
		idx = self.anifact.FindMorphTarget(target)
		if idx == InvalidStateID:
			return
		self.protected.append(idx)
	def _AddRandomMorphTarget(self,idx,name=None):
		def func():
			print " * run morph",idx,name
			self.SetMorphTargetIdx(idx)
 		return func
	def RandomMorphTargets(self,section,targets,mintime,maxtime):
		functions = []
		for target in targets:
			idx = self.anifact.FindMorphTarget(target)
			if idx == InvalidStateID:
				continue
			functions.append(self._AddRandomMorphTarget(idx,target))
		action = RandomAction(functions,mintime,maxtime)
		ActionsManager.append(action,section)
	def ClearRandomMorphTargets(self,section):
		ActionsManager.clear(section)
	def _SetMorphTarget(self,idx,endval):
		currval = self.animesh.GetMorphTargetWeight(idx)
		if currval == endval:
			return
		def set_morph(val):
			self.animesh.SetMorphTargetWeight(idx,val)
		action = FadedAction(set_morph,currval,endval,1000)
		ActionsManager.append(action,"morph"+str(idx))
	def ClearMorphTargets(self):
		for idx in xrange(self.anifact.GetMorphTargetCount ()):
			self._SetMorphTarget(idx,0.0)
	def SetMorphTargetIdx(self,target_idx):
		if not target_idx in self.protected:
			for idx in xrange(self.anifact.GetMorphTargetCount ()):
				if not idx == target_idx and not idx in self.protected:
					self._SetMorphTarget(idx,0.0)
		self._SetMorphTarget(target_idx,1.0)
	def SetMorphTarget(self,name):
		target_idx = self.anifact.FindMorphTarget(name)
		if not target_idx == InvalidStateID:
			self.SetMorphTargetIdx(target_idx)
		else:
			Reporter.ReportError("animesh","morph target not found")
	def SetWeight(self,blendname,idx,w):
		blendnode = self.root.FindNode(blendname)
		if blendnode:
			node = blendnode.QueryInterface(iSkeletonBlendNode2)
			node.SetNodeWeight(idx,w)
	def PlayState(self,fsmname,statename):
		fsmnode = self.root.FindNode(fsmname)
		if not fsmnode:
			return
		fsmfact = fsmnode.GetFactory().QueryInterface(iSkeletonFSMNodeFactory2)
		fsm = fsmnode.QueryInterface(iSkeletonFSMNode2)
		stateidx = fsmfact.FindState(statename)
		if stateidx == InvalidStateID:
			return
		fsm.SwitchToState(stateidx)
		if not fsmnode:
			return
		fsmnode.Play()
	def Play(self,nodename,reset=False):
		node = self.root.FindNode(nodename)
		if node:
			node.Play()
	def Stop(self,nodename=None):
		if not nodename:
			self.root.Stop()
			return
		node = self.root.FindNode(nodename)
		if not node:
			return
		node.Stop()
		animnode = node.QueryInterface(iSkeletonAnimationNode2)
		if animnode:
			animnode.SetPlaybackPosition(0.0)
	def FindMeshWrapper(self,obj):
		if isinstance(obj,iMeshWrapper):
			return obj
		elif isinstance(obj,iCelEntity):
			return celMesh(obj).Mesh
		elif isinstance(obj,iPcMesh):
			return obj.Mesh
		elif isinstance(obj,str):
			ent = Entities[obj]
			if ent:
				return celMesh(ent).Mesh
			else:
				return Engine.Meshes[obj]

#am = AnimeshWrapper('camera')
#am.Play('Frankie_Walk')
