from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock
from amhelper import AnimeshWrapper

# fix interfaces
iSkeletonFSMNodeFactory2.GetNodeCount = lambda s: s.GetStateCount()
iSkeletonFSMNodeFactory2.GetNode = lambda s,idx: s.GetStateNode(idx)

nodetypes = [
 iSkeletonBlendNodeFactory2,
 iSkeletonPriorityNodeFactory2,
 iSkeletonRandomNodeFactory2,
 iSkeletonFSMNodeFactory2,
]

class ApricotAnimation(ApricotPropertyClass):
    """
    cel property class
    Handle animation graph..
    """

    def __init__(self,oreg):
        ApricotPropertyClass.__init__(self,oreg)
        self._animdict = {} # to know if an anim is a state or something else
        self._sockets = {}
    def GetNodeFactIface(self,node):
        nodefact = node.GetFactory()
        for nodetype in nodetypes:
            iface = nodefact.QueryInterface(nodetype)
            if iface:
                return iface
    def Stop(self,nodename):
        self.anim.Stop(nodename)
    def TraverseTree(self,node,parent=None,nodeidx=None): # :-P
        # 1- get factory interfaces
        name = node.GetFactory().GetNodeName()
        fact = self.GetNodeFactIface(node)
        if parent:
            parentfact = self.GetNodeFactIface(parent)
        # 2- check who's the parent to see how to run the node
        if not parent or not parentfact.__class__ == iSkeletonFSMNodeFactory2:
            #print " * node:",name
            def Run():
                node.Play()
            self._animdict[name] = Run
        else:
            #print " * state node:",name
            fsmparent = parent.QueryInterface(iSkeletonFSMNode2)
            def Run():
                fsmparent.SwitchToState(nodeidx)
                parent.Play()
            self._animdict[name] = Run
        # 3- recurse into childs
        if fact:
            for idx in xrange(fact.GetNodeCount()):
                childfact = fact.GetNode(idx)
                childname = childfact.GetNodeName()
                childnode = node.FindNode(childname)
                if not node == childnode:
                    self.TraverseTree(childnode,node,idx)
                else:
                    Reporter.ReportError("apricot.animation","you probably have duplicate names for node "+childname)
    def TickEveryFrame(self):
        # sync socketed objects
        for socket,mws in self._sockets.items():
            for mw in mws:
                skelmov = self.anim.skeleton.GetSceneNode()
                mov = self.mesh.Movable.Position
                wtf = self.anim.skeleton.GetTransformAbsSpace(socket)
                mw.Movable.Position = mov+wtf
                mw.Movable.UpdateMove()
    def DetachObject(self,mw):
        sockets = self._sockets.keys()
        # we may be removing objects!, so dont change how this is iterated.
        for socket in self._sockets.keys():
            objects = self._sockets[socket]
            if mw in objects:
                objects.remove(mw)
            if not objects:
                self._sockets.pop(socket)
        if not self._sockets:
            PhysicalLayer.RemoveCallbackEveryFrame(self,CEL_EVENT_PRE)
    def AttachEntity(self,args):
        """
        cel action
        @param entity: entity to attach
        @type entity: iCelEntity
        @param bone: bone to attach to
        @type bone: str
        """
        bone = args["bone"]
        entity = args["entity"]
        if isinstance(entity,str):
            entity = Entities[entity]
        mw = celMesh(entity).Mesh
        self.AttachObject(bone,entity)
    def AttachObject(self,bone,mw):
        boneid = self.anim.skeleton.GetFactory().FindBone(bone)
        if boneid == InvalidBoneID:
            Reporter.ReportError("apricot.animation","Bone doesnt exist "+bone)
            return
        if not self._sockets:
            PhysicalLayer.CallbackEveryFrame(self,CEL_EVENT_PRE)
        if boneid not in self._sockets:
            self._sockets[boneid] = []
        mw.pos = "bla"
        self._sockets[boneid].append(mw)
    def SetMorphTarget(self,*args):
        self.anim.SetMorphTarget(*args)
    def RandomMorphTargets(self,*args):
        self.anim.RandomMorphTargets(*args)
    def ClearRandomMorphTargets(self,*args):
        self.anim.ClearRandomMorphTargets(*args)
    def AddProtectedMorphTarget(self,*args):
        self.anim.AddProtectedMorphTarget(*args)
    def Play(self,animname):
        runfunc = self._animdict.get(animname,None)
        if runfunc:
            runfunc()
    def PlayAnimation(self,args):
        """
        cel action
        """
        self.Play(args["name"])
    def StopAnimation(self,args):
        """
        cel action
        """
        self.Stop(args["name"])

    def SetEntity(self,entity):
        ApricotPropertyClass.SetEntity(self,entity)
        if not entity: return
        self.mesh = celMesh(entity).Mesh
        self.anim = AnimeshWrapper(entity)
        self.TraverseTree(self.anim.root)
        self.Play("idle")

CEL_IMPLEMENT_FACTORY(ApricotAnimation,"apricot.animation")

