from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock

class Carry(ApricotPropertyClass):
    """
    cel property class
    Component for picking up and carry other entities.
    """

    def __init__(self, oreg):
        ApricotPropertyClass.__init__(self, oreg)
        # Carrying object pointer of False
        self.carrying = False
        #: cel property type:string rw
        #: Name of the bone to attach
        self.bone = "bonename"
        #self.SubscribeMessage("cel.input.interact.down",self.Throw)

    def SetEntity(self, entity):
        ApricotPropertyClass.SetEntity(self, entity)
        if not entity: return
        self.envsensor = pycel.celSensorEnvironment(entity)
        self.anim = None
        self.PropertyClassesHaveChanged()

    def PropertyClassesHaveChanged(self):
        if not self.anim:
            self.anim = pycel.celGetApricotAnimation(self.entity)

    def Start(self, args):
        if not self.carrying:
            if self.envsensor.target_entity:
                if self.envsensor.target_entity.HasClass(getid("carryable")):
                    self.carrying = self.envsensor.target_entity
                    Reporter.ReportDebug("apricot.carry", "Attaching object")
                    self.anim.AttachObject(self.bone, celMesh(self.envsensor.target_entity).Mesh)
                    self.anim.Play("catch")
                    self.anim.Play("carrywalk")
                    return True
            else:
                return False
        """
        else:
            if self.envsensor.target_entity:
                if self.envsensor.target_entity.HasClass(getid("carryable")):
                    return
            self.carrying = False
            #self.anim.DetachObject(celMesh(self.envsensor.target_entity).Mesh)
            Reporter.ReportDebug("apricot.carry", "Detaching object")
        """
        return False

    def Throw(self, direction):
        if self.carrying:
            # detach from skeleton
            self.anim.DetachObject(celMesh(self.carrying).Mesh)
            # dont sync any more
            mechobj = celMechanicsObject(self.carrying)
            bodysync = celPhysicsBodySync(self.carrying)
            PhysicalLayer.RemoveCallbackEveryFrame(bodysync, CEL_EVENT_PRE)
            Reporter.ReportDebug("apricot.carry", "make static!")
            # make static
            mechobj.MakeStatic(False)
            mechobj.Body.Position = celMesh(self.carrying).Mesh.Movable.Position
            fw = celMesh(self.entity).Mesh.Movable.FullTransform.This2Other(direction).Unit()
            mechobj.LinearVelocity = fw * 15.0
            self.carrying = False

CEL_IMPLEMENT_FACTORY(Carry,"apricot.carry")
