import pycel
from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock
import math

class Franky(ApricotPropertyClass):
    """
    cel property class
    Franky
    """

    def __init__(self, oreg):
        ApricotPropertyClass.__init__(self, oreg)
        self.SubscribeMessage("cel.timer.wakeup.frame", self.Animate)
        self.SubscribeMessage("cel.timer.timeout.leap", self.TimeoutLeap)
        self.SubscribeMessage("cel.timer.franky.brain", self.Brain)
        self.SubscribeMessage("apricot.sensor.environment.onground", self.SetAnimationLand)
        self.SubscribeMessage("cel.trigger.entity.enter", self.InTouch, "touch")
        self.SubscribeMessage("cel.trigger.entity.leave", self.OutTouch, "touch")
        self.SubscribeMessage("cel.trigger.entity.enter", self.InRange, "range")
        self.SubscribeMessage("cel.trigger.entity.leave", self.OutRange, "range")
        self.SubscribeMessage("cel.trigger.entity.enter", self.InView, "view")
        self.SubscribeMessage("cel.trigger.entity.leave", self.OutView, "view")
        self.SubscribeMessage("cel.input.camleft.down", self.PressCamLeft)
        self.SubscribeMessage("cel.input.camleft.up", self.ReleaseCamLeft)
        self.SubscribeMessage("cel.input.camright.down", self.PressCamRight)
        self.SubscribeMessage("cel.input.camright.up", self.ReleaseCamRight)
        self.SubscribeMessage("cel.input.camup.down", self.PressCamUp)
        self.SubscribeMessage("cel.input.camup.up", self.ReleaseCamUp)
        self.SubscribeMessage("cel.input.camdown.down", self.PressCamDown)
        self.SubscribeMessage("cel.input.camdown.up", self.ReleaseCamDown)
        self.SubscribeMessage("cel.entity.loaded", self.Setup)
        self.cdsys = oreg.Get(iCollideSystem)
        self.onground = False
        self.button_north = False
        self.button_south = False
        self.button_west = False
        self.button_east = False

    def SetEntity(self, entity):
        ApricotPropertyClass.SetEntity(self, entity)
        if not entity: return
        self.timer = celTimer(entity)
        self.quest = celQuest(entity)
        self.prop = celProperties(entity)
        self.mesh = celMesh(entity)
        self.movable = self.mesh.Mesh.GetMovable()
        self.linmove = celLinearMovement(entity)
        self.actoranalog = celAnalogMotion(entity)
        self.roll = pycel.celRoll(entity)
        self.carry = pycel.celCarry(entity)
        self.camera = celTrackingCamera(entity)
        self.velocity = csVector3(0)
        self.anim = None
        self.combo_jump = 0
        self.move_speed = 5.0
        self.moving_left = False
        self.moving_right = False
        self.moving_up = False
        self.moving_down = False
        self.rotating_left = False
        self.rotating_right = False
        self.sector = None
        self.trigger_touch = None
        self.trigger_range = None
        self.trigger_view = None
        self.leap = False
        self.PropertyClassesHaveChanged()
        self.timer.WakeUpFrame(CEL_EVENT_PRE)
        self.timer.WakeUp(100, True, "franky.brain")

    def PropertyClassesHaveChanged(self):
        if not self.sector:
            self.sector = self.movable.GetSectors()[0]
        if not self.anim:
            self.anim = pycel.celGetApricotAnimation(self.entity)
        if not self.trigger_touch:
            self.trigger_touch = celTrigger(self.entity, "touch")
            self.trigger_touch.SetupTriggerSphere(self.sector, csVector3(0,0,0), 0.3)
            self.trigger_touch.MonitorClass("interact")
            self.trigger_touch.follow = True
            self.trigger_touch.EnableTrigger(True)
        if not self.trigger_range:
            self.trigger_range = celTrigger(self.entity, "range")
            self.trigger_range.SetupTriggerSphere(self.sector, csVector3(0,0,0), 5.0)
            self.trigger_range.MonitorClass("interact")
            self.trigger_range.follow = True
            self.trigger_range.EnableTrigger(True)
        if not self.trigger_view:
            self.trigger_view = celTrigger(self.entity, "view")
            self.trigger_view.SetupTriggerSphere(self.sector, csVector3(0,0,0), 10.0)
            self.trigger_view.MonitorClass("interact")
            self.trigger_view.follow = True
            self.trigger_view.EnableTrigger(True)

    def Setup(self, args):
        print "Setting up frankie"

    def Animate(self, args):
        #if self.leap:
        #    self.linmove.SetVelocity(csVector3(0.0, 0.0, -2.0))
        self.velocity = self.linmove.GetVelocity()
        #print "velocity=", self.velocity.x, self.velocity.y, self.velocity.z
        # Camera behaviour
        """
        if self.quest.state == "walk":
            if self.button_south:
                self.camera.posoff_angle = 0.2
                self.camera.posoff_dist = 8.0
                self.camera.targetyoffset = 0.5
            else:
                self.camera.posoff_angle = 0.4
                self.camera.posoff_dist = 6.0
                self.camera.targetyoffset = 1.0
        """
        if self.quest.state == "walk":
            #zprint "velocity =", (abs(self.velocity.x) + abs(self.velocity.z))
            if abs(self.velocity.y) < 0.01:
                # Idle
                if (abs(self.velocity.x) + abs(self.velocity.z)) < 0.01:
                    self.anim.Play("idle")
                    return
                # Run2Idle
                if (abs(self.velocity.x) + abs(self.velocity.z)) > 6.0 and self.button_north == False and self.button_south == False and self.button_east == False and self.button_west == False:
                    #self.anim.Play("run2idle")
                    return
                # Run
                if (abs(self.velocity.x) + abs(self.velocity.z)) > 6.0:
                    if self.button_north == True or self.button_south == True or self.button_east == True or self.button_west == True:
                        self.anim.Stop("run2idle")
                    self.anim.Play("run")
                    return
                # Run Slow
                if (abs(self.velocity.x) + abs(self.velocity.z)) > 5.0:
                    if self.button_north == True or self.button_south == True or self.button_east == True or self.button_west == True:
                        self.anim.Stop("run2idle")
                    self.anim.Play("runslow")
                    return
                # Walk Fast
                if (abs(self.velocity.x) + abs(self.velocity.z)) > 4.0:
                    if self.button_north == True or self.button_south == True or self.button_east == True or self.button_west == True:
                        self.anim.Stop("run2idle")
                    self.anim.Play("walkfast")
                    return
                # Walk
                if (abs(self.velocity.x) + abs(self.velocity.z)) > 2.0:
                    if self.button_north == True or self.button_south == True or self.button_east == True or self.button_west == True:
                        self.anim.Stop("run2idle")
                    self.anim.Play("walk")
                    return
                # Walk Slow
                if (abs(self.velocity.x) + abs(self.velocity.z)) > 0.01:
                    if self.button_north == True or self.button_south == True or self.button_east == True or self.button_west == True:
                        self.anim.Stop("run2idle")
                    self.anim.Play("walkslow")
                    return
            # Jump
            if self.velocity.y > 0.01:
                self.anim.Play("jump")
                return
            # Fall
            if self.velocity.y < -0.01:
                self.anim.Play("fall")
                return

    def SetAnimationLand(self, args):
        self.anim.Stop("jump")
        self.anim.Stop("fall")
        self.anim.Play("land")

    def Brain(self, args):
        self.onground = self.linmove.IsOnGround()
        if self.quest.state == "walk":
            ground_distance = self.SensorMesh(csVector3(0.0, 0.29, 0.25), csVector3(0.0, -100.0, 0.0))
            # Leap: when floating in the air with small distance from ground
            #if self.leap == False and self.onground == True and ground_distance > 0.5 and ground_distance < 10.0:
            #    self.leap = True
            #    self.timer.WakeUp(500, False, "timeout.leap")
            #    self.linmove.AddVelocity(csVector3(0.0, 2.0, 0.0))
            # Glide: when far distance from ground during jumping
            if self.onground == False and ground_distance > 5.0:
                self.quest.GetQuest().SwitchState("glide")

    def TimeoutLeap(self, args):
        self.leap = False

    def Interact(self, args):
        """
        cel action
        Interaction dispatcher to proper action
        """
        if self.carry.carrying:
            self.carry.Throw(csVector3(0.0, 1.0, 1.0))
            self.anim.Play("throw")
            return
        if self.carry.Start(None) == True:
            return
        self.entity.QueryMessageChannel().SendMessage("apricot.interact.throw", self, NoneParBlock)

    def Jump(self, args):
        """
        cel action
        Jumping dispatcher to proper action
        """
        self.onground = self.linmove.IsOnGround()
        if self.onground:
            self.linmove.AddVelocity(csVector3(0.0, 10.0, 0.0))
        self.quest.GetQuest().SwitchState("walk")

    def MoveLeft(self, args):
        """
        cel action
        Moving to left
        @param start: Start/Stop moving
        @type start: bool
        """
        start = args["start"]
        if start:
            if not self.moving_left:
                self.linmove.AddVelocity(csVector3(self.move_speed, 0.0, 0.0))
                self.moving_left = True
        else:
            if self.moving_left:
                self.linmove.AddVelocity(csVector3(-self.move_speed, 0.0, 0.0))
                self.moving_left = False

    def RotateLeft(self, args):
        """
        cel action
        Rotating to left
        @param start: Start/Stop rotating
        @type start: bool
        """
        start = args["start"]
        if start:
            if not self.rotating_left:
                self.actoranalog.SetAxis(0, -0.5)
                self.rotating_left = True
        else:
            if self.rotating_left:
                self.actoranalog.SetAxis(0, 0)
                self.rotating_left = False

    def MoveRight(self, args):
        """
        cel action
        Moving to right
        @param start: Start/Stop moving
        @type start: bool
        """
        start = args["start"]
        if start:
            if not self.moving_right:
                self.linmove.AddVelocity(csVector3(-self.move_speed, 0.0, 0.0))
                self.moving_right = True
        else:
            if self.moving_right:
                self.linmove.AddVelocity(csVector3(self.move_speed, 0.0, 0.0))
                self.moving_right = False

    def RotateRight(self, args):
        """
        cel action
        Rotating to right
        @param start: Start/Stop rotating
        @type start: bool
        """
        start = args["start"]
        if start:
            if not self.rotating_right:
                self.actoranalog.SetAxis(0, 0.5)
                self.rotating_right = True
        else:
            if self.rotating_right:
                self.actoranalog.SetAxis(0, 0)
                self.rotating_right = False

    def MoveUp(self, args):
        """
        cel action
        Moving up
        @param start: Start/Stop moving
        @type start: bool
        """
        start = args["start"]
        if start:
            if not self.moving_up:
                self.linmove.AddVelocity(csVector3(0.0, self.move_speed, 0.0))
                self.moving_up = True
        else:
            if self.moving_up:
                self.linmove.AddVelocity(csVector3(0.0, -self.move_speed, 0.0))
                self.moving_up = False

    def MoveDown(self, args):
        """
        cel action
        Moving down
        @param start: Start/Stop moving
        @type start: bool
        """
        start = args["start"]
        if start:
            if not self.moving_down:
                self.linmove.AddVelocity(csVector3(0.0, -self.move_speed, 0.0))
                self.moving_down = True
        else:
            if self.moving_down:
                self.linmove.AddVelocity(csVector3(0.0, self.move_speed, 0.0))
                self.moving_down = False

    def MoveStop(self, args):
        """
        cel action
        Stop moving
        """
        if self.moving_left:
            self.linmove.AddVelocity(csVector3(-self.move_speed, 0.0, 0.0))
            self.moving_left = False
        if self.rotating_left:
            self.actoranalog.SetAxis(0, 0)
            self.rotating_left = False
        if self.moving_right:
            self.linmove.AddVelocity(csVector3(self.move_speed, 0.0, 0.0))
            self.moving_right = False
        if self.rotating_right:
            self.actoranalog.SetAxis(0, 0)
            self.rotating_right = False
        if self.moving_up:
            self.linmove.AddVelocity(csVector3(0.0, self.move_speed, 0.0))
            self.moving_up = False
        if self.moving_down:
            self.linmove.AddVelocity(csVector3(0.0, -self.move_speed, 0.0))
            self.moving_down = False

    def InTouch(self, args):
        Reporter.ReportDebug("apricot.franky", "InTouch")
        args["entity"].MessageChannel.SendMessage("apricot.franky.intouch", self, NoneParBlock)

    def OutTouch(self, args):
        Reporter.ReportDebug("apricot.franky", "OutTouch")
        args["entity"].MessageChannel.SendMessage("apricot.franky.outtouch", self, NoneParBlock)

    def InRange(self, args):
        Reporter.ReportDebug("apricot.franky", "InRange")
        args["entity"].MessageChannel.SendMessage("apricot.franky.inrange", self, NoneParBlock)

    def OutRange(self, args):
        Reporter.ReportDebug("apricot.franky", "OutRange")
        args["entity"].MessageChannel.SendMessage("apricot.franky.outrange", self, NoneParBlock)

    def InView(self, args):
        Reporter.ReportDebug("apricot.actor.franky", "InView")
        args["entity"].MessageChannel.SendMessage("apricot.franky.inview", self, NoneParBlock)

    def OutView(self, args):
        Reporter.ReportDebug("apricot.actor.franky", "OutView")
        args["entity"].MessageChannel.SendMessage("apricot.franky.outview", self, NoneParBlock)

    def SensorMesh(self, bstart, bend):
        # Setting start and end vector
        beam_start = self.movable.GetFullPosition() + bstart
        beam_end = beam_start + (self.movable.GetFullTransform().This2Other(bend + bstart) - self.movable.GetFullTransform().This2Other(bstart))
        # Detecing meshes
        self.mesh.Mesh.SetFlagsRecursive(CS_ENTITY_NOHITBEAM)
        #hitbeam = self.meshwrapper.HitBeam(beam_start, beam_end)
        hitbeam = csColliderHelper.TraceBeam(self.cdsys, self.sector, beam_start, beam_end, False)
        self.mesh.Mesh.SetFlagsRecursive(CS_ENTITY_NOHITBEAM, 0)
        if hitbeam.closest_mesh:
            if hitbeam.sqdistance > 0.0:
                return math.sqrt(hitbeam.sqdistance)
            else:
                return -1.0
        else:
            return -1.0

    def PressCamLeft(self, args):
        self.camera.Pan(self.camera.PAN_LEFT)
        #self.camera.SetPanDirection(-1.0)

    def ReleaseCamLeft(self, args):
        self.camera.Pan(self.camera.PAN_NONE)
        #self.camera.SetPanDirection(0.0)

    def PressCamRight(self, args):
        self.camera.Pan(self.camera.PAN_RIGHT)
        #self.camera.SetPanDirection(1.0)

    def ReleaseCamRight(self, args):
        self.camera.Pan(self.camera.PAN_NONE)
        #self.camera.SetPanDirection(0.0)

    def PressCamUp(self, args):
        self.camera.Tilt(self.camera.TILT_UP)

    def ReleaseCamUp(self, args):
        self.camera.Tilt(self.camera.TILT_NONE)

    def PressCamDown(self, args):
        self.camera.Tilt(self.camera.TILT_DOWN)

    def ReleaseCamDown(self, args):
        self.camera.Tilt(self.camera.TILT_NONE)

CEL_IMPLEMENT_FACTORY(Franky,"apricot.actor.franky")
