from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock
import hud
from hud import CreateBillboard, BBSPACE

class HudInventory(ApricotPropertyClass,pyPcInventoryListener):
    """
    cel property class
    Inventory on HUD.
    """

    def __init__(self, oreg):
        ApricotPropertyClass.__init__(self, oreg)
        pyPcInventoryListener.__init__(self, oreg)

    def SetEntity(self, entity):
        ApricotPropertyClass.SetEntity(self, entity)
        if not entity: return
        self.inv = celInventory(entity)
        self.inv.AddInventoryListener(self)
        icon_acorn = "props_acorn.png"
        icon_chestnut = "props_chestnut.png"
        icon_peach = "props_peach.png"
        self.icons = {}
        acorn_bb = CreateBillboard(self.entity, icon_acorn, 0.95, 0.85, 0.1, 0.1)
        peach_bb = CreateBillboard(self.entity, icon_peach, 0.9, 0.9, 0.1, 0.1)
        chestnut_bb = CreateBillboard(self.entity, icon_chestnut, 0.9, 0.9, 0.1, 0.1)
        self.icons["chestnut"] = chestnut_bb
        self.icons["peach"] = peach_bb
        self.icons["acorn"] = acorn_bb
        for icon in self.icons.values():
            icon.visible = False
        self.Update()

    def AddChild(self,inv,ent):
        self.Update(None)

    def RemoveChild(self,inv,ent):
        self.Update(None)

    def Update(self,args=None):
        Reporter.ReportWarning("apricot.hud.inventory","entity added")
        acorns = 0
        chestnuts = 0
        peaches = 0
        thingtypes = ["acorn","chestnut","peach"]
        thingcount = {}
        # set counts to 0
        for thing in thingtypes:
            thingcount[thing] = 0
        # count number of items
        for idx in xrange(self.inv.GetEntityCount()):
            ent = self.inv.GetEntity(idx)
            for thing in thingtypes:
                if thing in ent.Name:
                    thingcount[thing]+=1
        # add to list stuff we have at least one
        curr_icons = []
        for thing in thingtypes:
            if thingcount[thing]:
                curr_icons.append(thing)
        # make all invisible
        for icon in hud.icons.values():
            icon.visible=False
            icon.text = ""
        # show correct ones
        for idx, iconname in enumerate(curr_icons):
            icon = self.icons[iconname]
            icon.x = int(BBSPACE * (0.85 - (idx * (0.07))))
            icon.visible = True
            if idx == 0:
                icon.text = str(thingcount[iconname])
                icon.text_offset = csVector2(BBSPACE * 0.1, BBSPACE * 0.025)
                icon.width = int(BBSPACE * 0.1)
                icon.height = int(BBSPACE * 0.1)
            else:
                icon.width = int(BBSPACE * (0.08 / idx))
                icon.height = int(BBSPACE * (0.08 / idx))

CEL_IMPLEMENT_FACTORY(HudInventory,"apricot.hud.inventory")

