from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock
import random
myrandom = lambda s: (random.random()-0.5)*s*2

class MoveGround(ApricotPropertyClass):
    """
    cel property class
    Moving AI on navigation graph specific for ground.
    """

    def __init__(self, oreg):
        ApricotPropertyClass.__init__(self, oreg)
        #: cel property type:string rw
        #: Name of the navigation graph
        self.graph = "navi_ground"
        #: cel property type:string rw
        #: Name of the entity class to cohesion
        self.cohesion_class = ""
        #: cel property type:string rw
        #: Name of the entity class to separation for
        self.separation_class = ""
        self.SubscribeMessage("cel.timer.move.ground.brain", self.Brain)
        self.SubscribeMessage("cel.entity.loaded", self.Setup)
        self.escape_ent = None

    def SetEntity(self, entity):
        ApricotPropertyClass.SetEntity(self, entity)
        if not entity: return
        self.timer = celTimer(entity)
        self.mesh = celGetMesh(entity)
        self.meshwrapper = self.mesh.Mesh
        self.movable = self.meshwrapper.GetMovable()
        self.sector = None
        self.pathfinder = celPathFinder(self.entity)
        self.steer = celSteer(entity)
        self.timer.WakeUp(2000, True, "move.ground.brain")
        self.PropertyClassesHaveChanged()

    def PropertyClassesHaveChanged(self):
        return
        self.sector = self.movable.GetSectors()[0]
        igraph = self.sector.Object.GetChild(iCelGraph, self.graph)
        if not igraph:
            Reporter.ReportWarning("apricot.move.ground", "No navigation graph!")
            return
        self.pathfinder.SetGraph(igraph)
        self.steer.CheckArrivalOn(1.0)
        self.steer.SetDelayRecheck(int(1000 + (random.random() * 100)))
        #self.steer.collision_avoidance = True
        self.steer.Vigilant()

    def Setup(self, args):
        pass

    def Brain(self, args = None):
        self.steer.CohesionOn(PhysicalLayer.GetClassEntitiesList(getid(self.cohesion_class)), 15.0, 80.0, 0.5)
        self.steer.SeparationOn(PhysicalLayer.GetClassEntitiesList(getid(self.separation_class)), 3.0, 1.0)
        self.UpdateEscape()
        if not self.steer.moving:
            self.Wander()

    def Wander(self, args = None):
        """
        cel action
        """
        self.escape_ent = None
        #self.pathfinder.Wander(10)

    def Pursue(self, args = None):
        """
        cel action
        @param entity: chased entity name
        @type entity: string
        """
        self.sector = self.movable.GetSectors()[0]
        target_ent = Entities[args["entity"]]
        #if target_ent:
        #    self.pathfinder.Pursue(target_ent, 0)

    def Escape(self, args = None):
        """
        cel action
        @param entity: escaping from entity name
        @type entity: string
        """
        self.escape_ent = Entities[args["entity"]]
        self.UpdateEscape()

    def UpdateEscape(self):
        if self.escape_ent:
            self.sector = self.movable.GetSectors()[0]
            target_pos = celMesh(self.escape_ent).fullposition
            if target_pos:
                self.steer.Flee(self.sector, target_pos)

    def Stop(self, args = None):
        """
        cel action
        """
        #self.pathfinder.Interrupt()
        self.steer.Interrupt()
        self.escape_ent = None

CEL_IMPLEMENT_FACTORY(MoveGround,"apricot.move.ground")
