from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock

class NpcTutorial(ApricotPropertyClass):
    """
    cel property class
    Class used in tutorial only.
    """

    def __init__(self, oreg):
        ApricotPropertyClass.__init__(self, oreg)

    def SetEntity(self, entity):
        ApricotPropertyClass.SetEntity(self, entity)
        if not entity: return
        self.mesh = celGetMesh(entity)
        self.meshwrapper = self.mesh.Mesh
        self.movable = self.meshwrapper.GetMovable()
        self.pathfinder = celPathFinder(self.entity)
        self.steer = celSteer(entity)
        self.PropertyClassesHaveChanged()

    def PropertyClassesHaveChanged(self):
        self.sector = self.movable.GetSectors()[0]
        navigraph = self.sector.Object.GetChild(iCelGraph, "navigraph")
        if not navigraph: return
        self.pathfinder.SetGraph(navigraph)
        self.steer.CheckArrivalOn(1.0)

    def PrintHelloWorld(self, args):
        """
        cel action
        Printing 'Hello world' on text console.
        """
        print "Hello world"

    def Print(self, args):
        """
        cel action
        Printing custom text on console.
        @param text: Text for displaying
        @type text: string
        """
        print args["text"]

    def Wander(self, args):
        """
        cel action
        Wandering on navigation graph.
        """
        if self.pathfinder:
            self.pathfinder.Wander(100)

    def Pursue(self, args):
        """
        cel action
        @param entity: chased entity name
        @type entity: string
        """
        print "pursue"
        target_ent = Entities[args["entity"]]
        if target_ent:
            print "pursueok"
            self.pathfinder.Pursue(target_ent, 0)

CEL_IMPLEMENT_FACTORY(NpcTutorial,"mygame.npctutorial")
