import pycel
from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock

class PhysicsPush(ApricotPropertyClass):
    """
    cel property class
    Pushing physic entities by non-physic entity.
    """

    def __init__(self,oreg):
        ApricotPropertyClass.__init__(self,oreg)
        self._prevpush = None
        #TODO: it's hardcoded to messages, make it universal
        self.SubscribeMessage("cel.input.north.down",self.StartPush)
        self.SubscribeMessage("cel.input.south.down",self.StartPush)
        self.SubscribeMessage("cel.input.east.down",self.StartPush)
        self.SubscribeMessage("cel.input.west.down",self.StartPush)
        self.SubscribeMessage("cel.input.north.up",self.StopPush)
        self.SubscribeMessage("cel.input.south.up",self.StopPush)
        self.SubscribeMessage("cel.input.east.up",self.StopPush)
        self.SubscribeMessage("cel.input.west.up",self.StopPush)
        self.SubscribeMessage("cel.timer.physics.push",self.Push)
        #: cel property type:float
        #: Force
        self.force = 20.0

    def SetEntity(self,entity):
        ApricotPropertyClass.SetEntity(self,entity)
        if not entity:
            self.mesh = None
            self.movable = None
            self._entity = None
            return
        self.mesh = celGetMesh(entity)
        self.movable = self.mesh.Mesh.GetMovable()
        self.envsensor = pycel.celSensorEnvironment(entity)
        self.timer = celTimer(entity)

    def StartPush(self,args):
        self.timer.WakeUp(100, True, "physics.push")

    def Push(self,args):
        fw = self.movable.GetFullTransform().GetFront()
        direction = -1 * fw.Unit() #TODO: change after Forward direction bug fix
        if not self._prevpush == None:
            self._prevpush[0].RemoveForceTagged(self._prevpush[1])
        #Reporter.ReportDebug("apricot.phx_push","push!" + str(Clock.CurrentTicks / 1000))
        if not self.envsensor.target_entity:
            return
        mechobj = celGetMechanicsObject(self.envsensor.target_entity)
        if not mechobj:
            return
        mechobj.GetBody().Enable()
        self._prevpush = [mechobj, mechobj.AddForceTagged(direction * self.force, 0, csVector3(0))]

    def StopPush(self,args):
        if not self._prevpush == None:
            self._prevpush[0].RemoveForceTagged(self._prevpush[1])
            self._prevpush = None
        self.timer.Clear("physics.push")

CEL_IMPLEMENT_FACTORY(PhysicsPush,"apricot.physics.push")
