from pycel import *
import csproperties
from apricotpc import ApricotPropertyClass, NoneParBlock

class PhysicsThrow(ApricotPropertyClass):
    """
    cel property class
    Throwing objects from inventory using physics.
    Allows to throw an object of a certain class, with a given delay.
    """

    def __init__(self, oreg):
        ApricotPropertyClass.__init__(self,oreg)
        self.SubscribeMessage("apricot.interact.throw",self.Allow)
        self.SubscribeMessage("cel.timer.physics.throw",self.Throw)
        #: cel property type:string
        #: Type of entities that can be thrown
        self.projectileclass = "throwable"
        #: cel property type:int
        #: Time to wait from start throw to real throw
        #: (to sync with animation)
        self.delay_start = 200
        #: cel property type:int
        #: Time to wait from end throw to next throw
        self.delay_end = 1000
        #: cel property type:float
        #: Arm offset from origin
        self.armoffset = 0.5
        #: cel property type:vector3
        #: Throw direction vector
        self.direction = csVector3(0.0, 0.0, 0.0)
        #: cel property type:vector3
        #: Throw angle vector
        self.angle = csVector3(0.0, 0.0, 0.0)
        #: cel property type:float
        #: Throwing strength
        self.strength = 15.0
        # throwed 0=allowed 1=throwed 2=finished
        self.throwed = 0

    def SetEntity(self, entity):
        ApricotPropertyClass.SetEntity(self,entity)
        if not entity: return
        self.mov = celGetMesh(entity).Mesh.GetMovable()
        self.linmove = celLinearMovement(entity)
        self.prop = celProperties(entity)
        self.inventory = celInventory(entity)
        self.mesh = celGetMesh(entity)
        self.timer = celTimer(entity)
        self.anim = pycel.celApricotAnimation(entity)

    def Allow(self, args=None):
        # Picking projectile from inventory
        childidx = self.inventory.FindEntity(getid(self.projectileclass))
        if childidx == csArrayItemNotFound:
            return
        if self.throwed == 0:
            self.throwed = 1
            self.anim.Play("throw")
            self.timer.WakeUp(self.delay_start, False, "physics.throw")

    def Throw(self, args):
        if self.throwed == 1:
            childidx = self.inventory.FindEntity(getid(self.projectileclass))
            childentity = self.inventory.GetEntity(childidx)
            self.inventory.RemoveEntity(childentity)
            # Placing projectile in front of player
            childentity_mesh = celGetMesh(childentity)
            childentity_mesh.MoveMesh(self.mesh, csVector3(0.0, self.armoffset, 0.0))
            childentity_mesh.Mesh.SetFlagsRecursive(CS_ENTITY_INVISIBLEMESH, 0)
            # Throwing
            # TODO: after fixing to Front to proper direction remove minus
            front = -self.mesh.Mesh.Movable.GetTransform().GetFront()
            pcmech = celMechanicsObject(childentity)
            pcmech.Body.Position = childentity_mesh.Mesh.Movable.Position + csVector3(front.x, front.y + self.armoffset, front.z)
            pcmech.Body.AngularVelocity = self.angle
            pcmech.Body.LinearVelocity = (front + self.direction) * self.strength
            self.timer.WakeUp(self.delay_end, False, "physics.throw")
            self.throwed = 2
        elif self.throwed == 2:
            self.throwed = 0

CEL_IMPLEMENT_FACTORY(PhysicsThrow,"apricot.physics.throw")
