from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock
import sys
starttick = lambda self: PhysicalLayer.CallbackEveryFrame(self,CEL_EVENT_PRE)
endtick = lambda self: PhysicalLayer.RemoveCallbackEveryFrame(self,CEL_EVENT_PRE)
leader = None
import math
import random
myrandom = lambda s: (random.random()-0.5)*s*2

class ApricotRam(ApricotPropertyClass):
    """
    cel property class
    Pushing physic entities by non-physic entity.
    """
    def __init__(self,oreg):
        ApricotPropertyClass.__init__(self,oreg)
        #self.SubscribeMessage("apricot.franky.inrange",self.EnemyInRange)
        self.SubscribeMessage("cel.trigger.entity.enter.this",self.EnemyInView,"view")
        self.SubscribeMessage("cel.trigger.entity.leave.this",self.EnemyOutOfView,"view")
        self.SubscribeMessage("cel.trigger.entity.enter.this",self.EnemyInRange,"range")
        self.SubscribeMessage("cel.trigger.entity.leave.this",self.EnemyOutOfRange,"range")
        self.SubscribeMessage("cel.trigger.entity.enter.this",self.EnemyInTouch,"touch")

        #: cel property type:int
        #: time entity takes to look around
        self.thinktime = 700
        #: cel property type:string
        #: a navigation graph for the underlying pathfinder
        self.navigation_graph = "navigation_graph"
        self.enemy = None
        self.chargepos = None
    def ToWorld(self,*vector):
        return self.mov.GetTransform().This2Other(csVector3(*vector))
    def EnemyInTouch(self,args):
        Reporter.ReportDebug("apricot.ram","enemy in touch...!")
        self.am.Forward(False)
        celActorMove(self.enemy).Jump()
        celProperties(self.enemy)["life"]-=3
    def EnemyOutOfRange(self,args):
        Reporter.ReportDebug("apricot.ram","enemy out of range...!")
        self.anim.Play("idle")
    def TickOnce(self):
        self.EnemyInRange(None)
    def IsInFront(self,ownmov,othermov):
        fw = -ownmov.GetTransform().GetFront()
        fw.y = 0
        relpos = (othermov.Position - ownmov.Position)
        relpos.y = 0
        relpos = csYRotMatrix3(-math.pi/2.0)*relpos
        relpos.Normalize()
        if fw.Cross(relpos).y < 0:
            return True
        return False
    def TickEveryFrame(self):
        self.FaceEnemy()
        if self.chargepos:
            self.ResolveCharge()
    def FaceEnemy(self):
        enemy = self.enemy
        selffw = -self.mov.GetTransform().GetFront()
        selfpos3d = self.mov.Position
        enemypos3d = celMesh(enemy).Mesh.Movable.Position
        enemyrelpos = enemypos3d-selfpos3d
        selfpos2d = csVector2(selffw.x,selffw.z)
        enemypos2d = csVector2(enemyrelpos.x,enemyrelpos.z)
        selfpos2d /= selfpos2d.Norm()
        enemypos2d /= enemypos2d.Norm()
        sp = selfpos2d
        ep = enemypos2d
        angle = math.atan2(-ep.x,-ep.y)
        self.am.RotateTo(angle)
    def EnemyInView(self,args):
        self.enemy = args["entity"]
        self.anim.Play("walk")
        PhysicalLayer.CallbackEveryFrame(self,CEL_EVENT_PRE)
        self.FaceEnemy()
        Reporter.ReportDebug("apricot.ram","enemy in view")
    def EnemyOutOfView(self,args):
        PhysicalLayer.RemoveCallbackEveryFrame(self,CEL_EVENT_PRE)
        Reporter.ReportDebug("apricot.ram","enemy out view")
        self.am.Forward(False)
        self.chargepos = None
        self.chargedist = None
    def EnemyInRange(self,args):
        if not self.enemy:
            enemy = args["entity"]
        else:
            enemy = self.enemy
        self.Charge()
    def Charge(self,args=None):
        enemypos = celMesh(self.enemy).Mesh.Movable.Position
        selfpos = self.mov.Position
        self.chargedist = (enemypos-selfpos).SquaredNorm()*0.2
        self.chargepos = selfpos
        Reporter.ReportDebug("apricot.ram","enemy in range... "+str(self.enemy.Name))
        self.am.Forward(True)
        self.anim.Play("run")
    def ResolveCharge(self):
        selfpos = self.mov.Position
        dist = (self.chargepos - selfpos).SquaredNorm()
        if dist > self.chargedist:
             self.chargepos = False
             PhysicalLayer.RemoveCallbackEveryFrame(self,CEL_EVENT_PRE)
             self.chargedist = False
             self.am.Forward(False)
    def SetEntity(self,entity):
        ApricotPropertyClass.SetEntity(self,entity)
        if entity:
            entity.AddClass(getid("ram"))
            # setup actormove
            self.am = celActorMove(entity)
            self.anim = celApricotAnimation(entity)
            #am.automatic_animation = False
            #am.SetAnimationMapping(CEL_ANIM_IDLE,"sheepcute_graze")
            #am.SetAnimationMapping(CEL_ANIM_WALK,"sheepcute_run")
            #am.SetAnimationMapping(CEL_ANIM_RUN,"sheepcute_walk")
            self.am.MovementSpeed = 4
            self.am.RotationSpeed = 0.5
            # setup steer
            steer = celSteer(entity)
            self._steer = steer
            self.mov = celMesh(entity).Mesh.Movable
        else:
            self.mov = None

CEL_IMPLEMENT_FACTORY(ApricotRam,"apricot.npcs.ram")

