from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock

class SensorVelocity(ApricotPropertyClass):
    """
    cel property class
    Sensor for tracking velocity changes both for standard and physics environment (autodetect).
    @celmessage apricot.sensor.velocity.low: low velocity
    @celmessage apricot.sensor.velocity.high: high velocity
    """

    def __init__(self, oreg):
        ApricotPropertyClass.__init__(self, oreg)
        self.SubscribeMessage("cel.timer.sensor.velocity", self.VelocityRadar)
        #: cel property type:float
        #: Lowest values than ceiling are recognized as low other as high velocity
        self.ceiling = 1.0

    def SetEntity(self, entity):
        ApricotPropertyClass.SetEntity(self, entity)
        if not entity: return
        self.timer = celTimer(entity)
        self.linmove = celLinearMovement(entity)
        self.mechobj = celMechanicsObject(entity)
        self.value = 0.0
        self.std_velocity = csVector3(0)
        self.phx_velocity = csVector3(0)
        self.timer.WakeUp(250, True, "sensor.velocity")

    def VelocityRadar(self, args):
        if self.linmove:
            self.std_velocity = self.linmove.Velocity
        if self.mechobj:
            self.phx_velocity = self.mechobj.GetLinearVelocity()
        self.value = abs(self.std_velocity.x) + abs(self.std_velocity.y) + abs(self.std_velocity.z) + abs(self.phx_velocity.x) + abs(self.phx_velocity.y) + abs(self.phx_velocity.z)
        if self.value < self.ceiling:
            self.SendMessage("apricot.sensor.velocity.low")
        else:
            self.SendMessage("apricot.sensor.velocity.high")

CEL_IMPLEMENT_FACTORY(SensorVelocity,"apricot.sensor.velocity")

