from pycel import *
from apricotpc import ApricotPropertyClass, NoneParBlock
import sys
starttick = lambda self: PhysicalLayer.CallbackEveryFrame(self,CEL_EVENT_PRE)
endtick = lambda self: PhysicalLayer.RemoveCallbackEveryFrame(self,CEL_EVENT_PRE)
leader = None
import math
import random
myrandom = lambda s: (random.random()-0.5)*s*2

class ApricotSheep(ApricotPropertyClass):
    """
    cel property class
    Pushing physic entities by non-physic entity.
    """
    def __init__(self,oreg):
        ApricotPropertyClass.__init__(self,oreg)
        self.SubscribeMessage("cel.input.test.down",self.SeekShepherd)
        self.SubscribeMessage("cel.trigger.entity.enter.this",self.EnemyInView,"view")
        self.SubscribeMessage("cel.trigger.entity.leave.this",self.EnemyOutOfView,"view")
        self.SubscribeMessage("cel.trigger.entity.enter.this",self.EnemyInRange,"range")
        self.SubscribeMessage("cel.trigger.entity.leave.this",self.EnemyOutOfRange,"range")
        self.SubscribeMessage("cel.trigger.entity.enter.this",self.EnemyInTouch,"touch")
        self.SubscribeMessage("cel.mechanics.collision",self.Collide)
        #: cel property type:int
        #: time entity takes to look around
        self.thinktime = 700
        #: cel property type:string
        #: a navigation graph for the underlying pathfinder
        self.navigation_graph = "navigation_graph"
        self.enemy = None
    def DummyCollide(self,args):
        pass
    def Stop(self,args=None):
        s = celMesh(self.entity).Mesh.Movable.Sectors[0]
        self._steer.Seek(s,csVector3(myrandom(100.0),10,myrandom(100.0)))
        self._steer.Interrupt()
    def Collide(self,args):
        # XXX set seek first to be able to stop vigilant
        self.Stop()
        Reporter.ReportWarning("apricot.sheep","OUCH!")
        # XXX weakref was copied!!!
        self.UnsubscribeMessage("cel.trigger.entity.enter.this",self.EnemyInRange)
        self.UnsubscribeMessage("cel.input.test.down",self.SeekShepherd)
        self.UnsubscribeMessage("cel.mechanics.collision",self.Collide)
        celMesh(self.entity).SetAnimation("die",False)
    def SeekShepherd(self,args):
        s = celMesh(self.entity).Mesh.Movable.Sectors[0]
        graph = s.Object.GetChild(iCelGraph,"navigation_graph")
        if not graph:
            Reporter.ReportWarning("apricot.sheep","No navigation_graph!")
            return
        Reporter.ReportWarning("apricot.sheep","going there!!")
        self._pathfinder = celPathFinder(self.entity)
        self._pathfinder.SetGraph(graph)
        self._steer.CohesionOff()
        playerpos = celMesh(Entities.camera).position
        self._pathfinder.Seek(s,playerpos)
    def DisableTickOnce(self):
        PhysicalLayer.RemoveCallbackOnce(self,CEL_EVENT_PRE)
    def EnableTickOnce(self):
        PhysicalLayer.CallbackOnce(self,self.thinktime,CEL_EVENT_PRE)
    def EnemyOutOfRange(self,args):
        self.enemy = None
        self.DisableTickOnce()
    def EnemyOutOfView(self,args):
        pass
    def EnemyInView(self,args):
        pass
    def EnemyInTouch(self,args):
        pass
    def TickOnce(self):
        self.EnemyInRange(None)
    def IsInFront(self,ownmov,othermov):
        fw = -ownmov.GetTransform().GetFront()
        fw.y = 0
        relpos = (othermov.Position - ownmov.Position)
        relpos.y = 0
        relpos = csYRotMatrix3(-math.pi/2.0)*relpos
        relpos.Normalize()
        if fw.Cross(relpos).y < 0:
            return True
        return False
    def EnemyInRange(self,args):
        Reporter.ReportWarning("apricot.sheep","enemy in range")
        if not self.enemy:
            enemy = args["entity"]
        else:
            enemy = self.enemy
        ownmov = celMesh(self.entity).Mesh.Movable
        othermov = celMesh(enemy).Mesh.Movable
	if self.IsInFront(ownmov,othermov):
            print "in front"
            if not self.enemy:
                self.SubscribeMessage("cel.trigger.entity.leave.this",self.EnemyOutOfRange)
                self.enemy = enemy
            self.Panic()
            self.EnableTickOnce()
            return
        if self.enemy:
            #self.UnsubscribeMessage("cel.trigger.entity.leave.this",self.EnemyOutOfRange) XXX weakref copied!!!
            self.enemy = None
        s = ownmov.Sectors[0]
        Reporter.ReportWarning("apricot.sheep","enemy in range2")
        graph = s.Object.GetChild(iCelGraph,"navigation_graph")
        if not graph:
            Reporter.ReportWarning("apricot.sheep","No navigation_graph!")
            return
        self._pathfinder = celPathFinder(self.entity)
        self._pathfinder.SetGraph(graph)
        self._steer.CohesionOff()
        self._steer.SeparationOff()
        r=self._pathfinder.Seek(s,csVector3(myrandom(100.0),10,myrandom(100.0)))
        celMesh(self.entity).SetAnimation("run",True)
        print "SEEK",r
        #self._pathfinder.Wander(50)
    def SetupLeader(self):
        global leader
        leader = self.entity
    def SetEntity(self,entity):
        ApricotPropertyClass.SetEntity(self,entity)
        if entity:
            if not leader:
                self.SetupLeader()
                input = celCommandInput(Entities.camera)
                input.Bind("t","test")
            entity.AddClass(getid("interact"))
            entity.AddClass(getid("sheep"))
            # setup actormove
            am = celActorMove(entity)
            #am.automatic_animation = False
            #am.SetAnimationMapping(CEL_ANIM_IDLE,"sheepcute_graze")
            am.SetAnimationMapping(CEL_ANIM_WALK,"run")
            am.SetAnimationMapping(CEL_ANIM_RUN,"walk")
            am.MovementSpeed = 0.5
            am.RotationSpeed = 0.5
            # setup steer
            steer = celSteer(entity)
            self._steer = steer
            steer.CheckArrivalOn(3)
            steer.SetDelayRecheck(int(1000+(random.random()*100)))
            steer.CohesionOn(pl.GetClassEntitiesList(getid("sheep")),15.0,80.0,0.5)
            steer.SeparationOn(pl.GetClassEntitiesList(getid("sheep")),3.0,1.0)
    def Panic(self,args=None):
            print "PANIC"
            steer = self._steer
            steer.CheckArrivalOn(3)
            steer.SetDelayRecheck(int(1000+(random.random()*100)))
            steer.Vigilant()
            #steer.CollisionAvoidanceOn(1.0,0.2)
            steer.CohesionOn(pl.GetClassEntitiesList(getid("sheep")),15.0,80.0,0.5)
            steer.SeparationOn(pl.GetClassEntitiesList(getid("sheep")),3.0,1.0)

CEL_IMPLEMENT_FACTORY(ApricotSheep,"apricot.npcs.sheep")
